package cat.hack3.codingtests.marsrover.core;

import java.util.List;
import java.util.logging.Logger;

/**
 * Notice that the representation is the following:
 * Latitude is Y, starts from 1 on the top and goes down incrementally
 * Longitude is X, starts from 1 on the left and goes to right incrementally
 *     1    2    3
 * 1  1-1  1-2  1-3
 * 2  2-1  2-2  2-3
 * 3  3-1  3-2  3-3
 */
public class MarsMap {
    Logger logger = Logger.getLogger(this.getClass().getName());
    private static final int FIRST_POSITION_IN_MAP = 1;
    private static final int INCREMENT_UNIT = 1;

    private final int height;
    private final int width;
    private Coordinates currentPosition;

    private final MapIncrementalPositionResolver positionResolver;

    public MarsMap(int height, int width, Coordinates startingCoordinates, List<Coordinates>... obstaclesLocalizations) {
        this.height = height;
        this.width = width;
        currentPosition = startingCoordinates;

        positionResolver = new MapIncrementalPositionResolver(FIRST_POSITION_IN_MAP, INCREMENT_UNIT);
    }

    public Coordinates updatePositionTowards(Direction direction) {
        switch (direction) {
            case NORTH -> decrementLatitude();
            case SOUTH -> incrementLatitude();
            case WEST -> decrementLongitude();
            case EAST -> incrementLongitude();
        }
        logger.info(String.format("Updated coordinates towards %s, now is %d-%d",
                direction, currentPosition.latitude(), currentPosition.longitude()));

        return currentPosition;
    }

    private void incrementLatitude() {
        int latitude = currentPosition.latitude();
        int newLatitude = positionResolver.getIncrementedPosition(latitude, height);
        setNewLatitude(newLatitude);
    }

    private void decrementLatitude() {
        int latitude = currentPosition.latitude();
        int newLatitude = positionResolver.getDecrementedPosition(latitude, height);
        setNewLatitude(newLatitude);
    }

    private void incrementLongitude() {
        int longitude = currentPosition.longitude();
        int newLongitude = positionResolver.getIncrementedPosition(longitude, width);
        setNewLongitude(newLongitude);
    }

    private void decrementLongitude() {
        int longitude = currentPosition.longitude();
        int newLongitude = positionResolver.getDecrementedPosition(longitude, width);
        setNewLongitude(newLongitude);
    }

    private void setNewLatitude(int newLatitude) {
        currentPosition = currentPosition.ofUpdatedLatitude(newLatitude);
    }

    private void setNewLongitude(int newLongitude) {
        currentPosition = currentPosition.ofUpdatedLongitude(newLongitude);
    }

    public Coordinates getCurrentPosition() {
        return currentPosition;
    }
}
